### Persefone.jl - a model of agricultural landscapes and ecosystems in Europe.
###
### This file holds the code for the Wyvern (https://en.wikipedia.org/wiki/Wyvern).
### NOT FOR ACTUAL USE! This is of course only a test species ;-)
### Thankfully, wyverns are not a species we have to manage for...
###

"""
Beware the wyvern! This evolutionary cousin of the dragon may only have two
legs, but that doesn't make it any less dangerous...
"""
@species Wyvern begin
    fecundity = 0.02
    maxage = 365
    speed = 20
    vision = 50
    aggression = 0.2
    huntsuccess = 0.8
end

"""
Wyverns are mythical creatures that require no special initialisation.
"""
@create Wyvern begin
    @debug "$(animalid(self)) created."
end

"""
Wyverns are ferocious hunters, scouring the landscape for their favourite
prey: wolpertingers...
"""
@phase Wyvern summer begin
    for a in @neighbours(self.speed, false)
        # check if a wolpertinger is in pouncing distance
        if typeof(a) == Wolpertinger
            @move(a.pos)
            if @rand() < self.huntsuccess
                @debug "$(animalid(self)) killed $(animalid(a))."
                @killother(a, 1.0, "predation")
                @goto reproduce
            else
                @debug "$(animalid(self)) failed to kill $(animalid(a))."
            end
        elseif typeof(a) == Wyvern && @rand() < self.aggression
            # wyverns also fight against each other if they get too close
            @move(a.pos)
            outcome = @rand()
            if outcome < 0.4
                @debug "$(animalid(self)) killed $(animalid(a)) in a fight."
                @killother(a, 1.0, "fighting")
            elseif outcome < 0.8
                @kill(1.0, "wounds sustained in a fight")
            end
            @goto reproduce
        end
    end
    # check if a wolpertinger is in seeing distance, or walk in a random direction
    direction = @randomdirection(self.speed)
    for a in @neighbours(self.vision)
        if typeof(a) == Wolpertinger
            direction = @directionto(a)
            break
        end
    end
    @walk(direction, self.speed)
    # reproduce every once in a blue moon
    @label reproduce
    (@rand() < self.fecundity) && @reproduce()
    # hibernate from November to March
    month(model.date) >= November && (@setphase(winter))
    (self.age == self.maxage) && @kill(1.0, "old age")
end

"""
Fortunately, wyverns hibernate in winter.
"""
@phase Wyvern winter begin
    # hibernate from November to March
    if month(model.date) >= March
        @setphase(summer)
    end
end

@populate Wyvern begin
    asexual = true
    popsize = 20
    initphase = winter
    birthphase = summer
    habitat = @habitat(@landcover() in (grass, soil, agriculture, builtup))
end
